@@ -1,16 +1,15 @@
-import sys
-
 import six
 
 try:
     from os import fsencode, fsdecode
 except ImportError:
-    from backports.os import fsencode, fsdecode
+    from backports.os import fsencode, fsdecode  # type: ignore
 
 try:
     from os import fspath
 except ImportError:
-    def fspath(path):
+
+    def fspath(path):  # type: ignore
         """Return the path representation of a path-like object.
 
         If str or bytes is passed in, it is returned unchanged. Otherwise the
@@ -27,14 +26,17 @@ except ImportError:
         try:
             path_repr = path_type.__fspath__(path)
         except AttributeError:
-            if hasattr(path_type, '__fspath__'):
+            if hasattr(path_type, "__fspath__"):
                 raise
             else:
-                raise TypeError("expected string type or os.PathLike object, "
-                                "not " + path_type.__name__)
+                raise TypeError(
+                    "expected string type or os.PathLike object, "
+                    "not " + path_type.__name__
+                )
         if isinstance(path_repr, (six.text_type, bytes)):
             return path_repr
         else:
-            raise TypeError("expected {}.__fspath__() to return string type "
-                            "not {}".format(path_type.__name__,
-                                            type(path_repr).__name__))
+            raise TypeError(
+                "expected {}.__fspath__() to return string type "
+                "not {}".format(path_type.__name__, type(path_repr).__name__)
+            )
