@@ -28,7 +28,7 @@ from .path import abspath, join, normpath
 from .time import datetime_to_epoch
 from .walk import Walker
 
-if False:  # typing.TYPE_CHECKING
+if typing.TYPE_CHECKING:
     from datetime import datetime
     from threading import RLock
     from typing import (
@@ -84,7 +84,7 @@ def _new_name(method, old_name):
 """.format(
         method.__name__
     )
-    if getattr(_method, "__doc__"):
+    if hasattr(_method, "__doc__"):
         _method.__doc__ += deprecated_msg
 
     return _method
@@ -175,7 +175,7 @@ class FS(object):
 
         This method will return a list of the resources in a directory.
         A *resource* is a file, directory, or one of the other types
-        defined in `~fs.ResourceType`.
+        defined in `~fs.enums.ResourceType`.
 
         Arguments:
             path (str): A path to a directory on the filesystem
@@ -823,11 +823,11 @@ class FS(object):
             path (str): A path on the filesystem.
 
         Returns:
-            ~fs.ResourceType: the type of the resource.
+            ~fs.enums.ResourceType: the type of the resource.
 
         A type of a resource is an integer that identifies the what
         the resource references. The standard type integers may be one
-        of the values in the `~fs.ResourceType` enumerations.
+        of the values in the `~fs.enums.ResourceType` enumerations.
 
         The most common resource types, supported by virtually all
         filesystems are ``directory`` (1) and ``file`` (2), but the
@@ -1624,7 +1624,8 @@ class FS(object):
 
         Arguments:
             path(str): A path on the filesystem.
-            name(str): One of the algorithms supported by the hashlib module, e.g. `"md5"`
+            name(str):
+                One of the algorithms supported by the hashlib module, e.g. `"md5"`
 
         Returns:
             str: The hex digest of the hash.
@@ -1633,7 +1634,7 @@ class FS(object):
             fs.errors.UnsupportedHash: If the requested hash is not supported.
 
         """
-        _path = self.validatepath(path)
+        self.validatepath(path)
         try:
             hash_object = hashlib.new(name)
         except ValueError:
