@@ -22,18 +22,20 @@ from .time import datetime_to_epoch
 from .errors import NoSysPath, MissingInfoNamespace
 from .walk import Walker
 
-if False:  # typing.TYPE_CHECKING
-    from typing import BinaryIO, Optional, Text, Tuple, Type, Union
+if typing.TYPE_CHECKING:
+    from typing import BinaryIO, Optional, Text, Tuple, Union
     from .base import FS
+
     ZipTime = Tuple[int, int, int, int, int, int]
 
 
-def write_zip(src_fs,                            # type: FS
-              file,                              # type: Union[Text, BinaryIO]
-              compression=zipfile.ZIP_DEFLATED,  # type: int
-              encoding="utf-8",                  # type: Text
-              walker=None                        # type: Optional[Walker]
-              ):
+def write_zip(
+    src_fs,  # type: FS
+    file,  # type: Union[Text, BinaryIO]
+    compression=zipfile.ZIP_DEFLATED,  # type: int
+    encoding="utf-8",  # type: Text
+    walker=None,  # type: Optional[Walker]
+):
     # type: (...) -> None
     """Write the contents of a filesystem to a zip file.
 
@@ -52,38 +54,29 @@ def write_zip(src_fs,                            # type: FS
             you want to compress.
 
     """
-    _zip = zipfile.ZipFile(
-        file,
-        mode="w",
-        compression=compression,
-        allowZip64=True
-    )
+    _zip = zipfile.ZipFile(file, mode="w", compression=compression, allowZip64=True)
     walker = walker or Walker()
     with _zip:
-        gen_walk = walker.info(
-            src_fs, namespaces=["details", "stat", "access"])
+        gen_walk = walker.info(src_fs, namespaces=["details", "stat", "access"])
         for path, info in gen_walk:
             # Zip names must be relative, directory names must end
             # with a slash.
-            zip_name = relpath(path + '/' if info.is_dir else path)
+            zip_name = relpath(path + "/" if info.is_dir else path)
             if not six.PY3:
                 # Python2 expects bytes filenames
-                zip_name = zip_name.encode(encoding, 'replace')
+                zip_name = zip_name.encode(encoding, "replace")
 
-            if info.has_namespace('stat'):
+            if info.has_namespace("stat"):
                 # If the file has a stat namespace, get the
                 # zip time directory from the stat structure
-                st_mtime = info.get('stat', 'st_mtime', None)
+                st_mtime = info.get("stat", "st_mtime", None)
                 _mtime = time.localtime(st_mtime)
                 zip_time = _mtime[0:6]  # type: ZipTime
             else:
                 # Otherwise, use the modified time from details
                 # namespace.
                 mt = info.modified or datetime.utcnow()
-                zip_time = (
-                    mt.year, mt.month, mt.day,
-                    mt.hour, mt.minute, mt.second
-                )
+                zip_time = (mt.year, mt.month, mt.day, mt.hour, mt.minute, mt.second)
 
             # NOTE(@althonos): typeshed's `zipfile.py` on declares
             #     ZipInfo.__init__ for Python < 3 ?!
@@ -98,29 +91,27 @@ def write_zip(src_fs,                            # type: FS
             if info.is_dir:
                 zip_info.external_attr |= 0x10
                 # This is how to record directories with zipfile
-                _zip.writestr(zip_info, b'')
+                _zip.writestr(zip_info, b"")
             else:
                 # Get a syspath if possible
                 try:
                     sys_path = src_fs.getsyspath(path)
                 except NoSysPath:
                     # Write from bytes
-                    _zip.writestr(
-                        zip_info,
-                        src_fs.getbytes(path)
-                    )
+                    _zip.writestr(zip_info, src_fs.readbytes(path))
                 else:
                     # Write from a file which is (presumably)
                     # more memory efficient
                     _zip.write(sys_path, zip_name)
 
 
-def write_tar(src_fs,            # type: FS
-              file,              # type: Union[Text, BinaryIO]
-              compression=None,  # type: Optional[Text]
-              encoding="utf-8",  # type: Text
-              walker=None        # type: Optional[Walker]
-              ):
+def write_tar(
+    src_fs,  # type: FS
+    file,  # type: Union[Text, BinaryIO]
+    compression=None,  # type: Optional[Text]
+    encoding="utf-8",  # type: Text
+    walker=None,  # type: Optional[Walker]
+):
     # type: (...) -> None
     """Write the contents of a filesystem to a tar file.
 
@@ -142,19 +133,14 @@ def write_tar(src_fs,            # type: FS
         ResourceType.directory: tarfile.DIRTYPE,
         ResourceType.fifo: tarfile.FIFOTYPE,
         ResourceType.file: tarfile.REGTYPE,
-        ResourceType.socket: tarfile.AREGTYPE,   # no type for socket
+        ResourceType.socket: tarfile.AREGTYPE,  # no type for socket
         ResourceType.symlink: tarfile.SYMTYPE,
         ResourceType.unknown: tarfile.AREGTYPE,  # no type for unknown
     }
 
-    tar_attr = [
-        ('uid', 'uid'),
-        ('gid', 'gid'),
-        ('uname', 'user'),
-        ('gname', 'group'),
-    ]
+    tar_attr = [("uid", "uid"), ("gid", "gid"), ("uname", "user"), ("gname", "group")]
 
-    mode = 'w:{}'.format(compression or '')
+    mode = "w:{}".format(compression or "")
     if isinstance(file, (six.text_type, six.binary_type)):
         _tar = tarfile.open(file, mode=mode)
     else:
@@ -163,21 +149,18 @@ def write_tar(src_fs,            # type: FS
     current_time = time.time()
     walker = walker or Walker()
     with _tar:
-        gen_walk = walker.info(
-            src_fs,
-            namespaces=["details", "stat", "access"]
-        )
+        gen_walk = walker.info(src_fs, namespaces=["details", "stat", "access"])
         for path, info in gen_walk:
             # Tar names must be relative
             tar_name = relpath(path)
             if not six.PY3:
                 # Python2 expects bytes filenames
-                tar_name = tar_name.encode(encoding, 'replace')
+                tar_name = tar_name.encode(encoding, "replace")
 
             tar_info = tarfile.TarInfo(tar_name)
 
-            if info.has_namespace('stat'):
-                mtime = info.get('stat', 'st_mtime', current_time)
+            if info.has_namespace("stat"):
+                mtime = info.get("stat", "st_mtime", current_time)
             else:
                 mtime = info.modified or current_time
 
@@ -191,8 +174,8 @@ def write_tar(src_fs,            # type: FS
                 if getattr(info, infoattr, None) is not None:
                     setattr(tar_info, tarattr, getattr(info, infoattr, None))
 
-            if info.has_namespace('access'):
-                tar_info.mode = getattr(info.permissions, 'mode', 0o420)
+            if info.has_namespace("access"):
+                tar_info.mode = getattr(info.permissions, "mode", 0o420)
 
             if info.is_dir:
                 tar_info.type = tarfile.DIRTYPE
