@@ -4,7 +4,6 @@
 from __future__ import print_function
 from __future__ import unicode_literals
 
-import collections
 import errno
 import platform
 import sys
@@ -15,9 +14,14 @@ from six import reraise
 
 from . import errors
 
-if False:  # typing.TYPE_CHECKING
+if typing.TYPE_CHECKING:
     from types import TracebackType
-    from typing import Iterator, Optional, Mapping, Text, Type, Union
+    from typing import Iterator, Optional, Text, Type, Union
+
+try:
+    from collections.abc import Mapping
+except ImportError:
+    from collections import Mapping  # noqa: E811
 
 
 _WINDOWS_PLATFORM = platform.system() == "Windows"
@@ -107,8 +111,8 @@ def unwrap_errors(path_replace):
         yield
     except errors.ResourceError as e:
         if hasattr(e, "path"):
-            if isinstance(path_replace, collections.Mapping):
+            if isinstance(path_replace, Mapping):
                 e.path = path_replace.get(e.path, e.path)
             else:
                 e.path = path_replace
-        reraise(type(e), e)
+        raise
