@@ -19,7 +19,6 @@ the expense of potentially copying extra files.
 from __future__ import print_function
 from __future__ import unicode_literals
 
-from contextlib import contextmanager
 import typing
 
 from ._bulk import Copier
@@ -29,7 +28,7 @@ from .opener import manage_fs
 from .tools import is_thread_safe
 from .walk import Walker
 
-if False:  # typing.TYPE_CHECKING
+if typing.TYPE_CHECKING:
     from typing import Callable, Optional, Text, Union
     from .base import FS
     from .info import Info
@@ -96,11 +95,7 @@ def mirror(
 
 
 def _mirror(
-    src_fs,
-    dst_fs,
-    walker=None,
-    copy_if_newer=True,
-    copy_file=copy_file_internal,
+    src_fs, dst_fs, walker=None, copy_if_newer=True, copy_file=copy_file_internal
 ):
     # type: (FS, FS, Optional[Walker], bool, Callable[[FS, str, FS, str], None]) -> None
     walker = walker or Walker()
@@ -108,8 +103,7 @@ def _mirror(
     for path, dirs, files in walk:
         try:
             dst = {
-                info.name: info
-                for info in dst_fs.scandir(path, namespaces=["details"])
+                info.name: info for info in dst_fs.scandir(path, namespaces=["details"])
             }
         except ResourceNotFound:
             dst_fs.makedir(path)
