@@ -15,8 +15,8 @@ import six
 from ._typing import Text
 
 
-if False:  # typing.TYPE_CHECKING
-    from typing import Container, FrozenSet, Set, Union
+if typing.TYPE_CHECKING:
+    from typing import FrozenSet, Set, Union
 
 
 __all__ = ["Mode", "check_readable", "check_writable", "validate_openbin_mode"]
@@ -67,6 +67,7 @@ class Mode(typing.Container[Text]):
         # type: (object) -> bool
         """Check if a mode contains a given character.
         """
+        assert isinstance(character, Text)
         return character in self._mode
 
     def to_platform(self):
