@@ -4,7 +4,6 @@
 from __future__ import print_function
 from __future__ import unicode_literals
 
-import copy
 import os
 import tarfile
 import typing
@@ -17,19 +16,19 @@ from . import errors
 from .base import FS
 from .compress import write_tar
 from .enums import ResourceType
+from .errors import IllegalBackReference, NoURL
 from .info import Info
 from .iotools import RawWrapper
 from .opener import open_fs
-from .path import dirname, relpath, basename, isbase, parts, frombase
-from .wrapfs import WrapFS
 from .permissions import Permissions
+from ._url_tools import url_quote
+from .path import relpath, basename, isbase, normpath, parts, frombase
+from .wrapfs import WrapFS
 
-
-if False:  # typing.TYPE_CHECKING
+if typing.TYPE_CHECKING:
     from tarfile import TarInfo
     from typing import (
         Any,
-        AnyStr,
         BinaryIO,
         Collection,
         Dict,
@@ -39,8 +38,7 @@ if False:  # typing.TYPE_CHECKING
         Tuple,
         Union,
     )
-    from .info import Info, RawInfo
-    from .permissions import Permissions
+    from .info import RawInfo
     from .subfs import SubFS
 
     T = typing.TypeVar("T", bound="ReadTarFS")
@@ -79,7 +77,7 @@ class TarFS(WrapFS):
     Here's how you might extract and print a readme from a tar file::
 
         with TarFS('foo.tar.gz') as tar_fs:
-            readme = tar_fs.gettext('readme.txt')
+            readme = tar_fs.readtext('readme.txt')
 
     If you open the TarFS with ``write`` set to `True`, then the TarFS
     will be a empty temporary filesystem. Any files / directories you
@@ -91,7 +89,7 @@ class TarFS(WrapFS):
     file::
 
         with TarFS('foo.tar.xz', write=True) as new_tar:
-            new_tar.settext(
+            new_tar.writetext(
                 'readme.txt',
                 'This tar file was written by PyFilesystem'
             )
@@ -143,7 +141,7 @@ class TarFS(WrapFS):
         else:
             return ReadTarFS(file, encoding=encoding)
 
-    if False:  # typing.TYPE_CHECKING
+    if typing.TYPE_CHECKING:
 
         def __init__(
             self,
@@ -272,9 +270,30 @@ class ReadTarFS(FS):
         else:
             self._tar = tarfile.open(fileobj=file, mode="r")
 
-        self._directory = OrderedDict(
-            (relpath(self._decode(info.name)).rstrip("/"), info) for info in self._tar
-        )
+        self._directory_cache = None
+
+    @property
+    def _directory_entries(self):
+        """Lazy directory cache."""
+        if self._directory_cache is None:
+            _decode = self._decode
+            _directory_entries = (
+                (_decode(info.name).strip("/"), info) for info in self._tar
+            )
+
+            def _list_tar():
+                for name, info in _directory_entries:
+                    try:
+                        _name = normpath(name)
+                    except IllegalBackReference:
+                        # Back references outside root, must be up to no good.
+                        pass
+                    else:
+                        if _name:
+                            yield _name, info
+
+            self._directory_cache = OrderedDict(_list_tar())
+        return self._directory_cache
 
     def __repr__(self):
         # type: () -> Text
@@ -318,7 +337,7 @@ class ReadTarFS(FS):
         else:
             try:
                 implicit = False
-                member = self._tar.getmember(self._encode(_path))
+                member = self._directory_entries[_path]
             except KeyError:
                 if not self.isdir(_path):
                     raise errors.ResourceNotFound(path)
@@ -361,14 +380,14 @@ class ReadTarFS(FS):
     def isdir(self, path):
         _path = relpath(self.validatepath(path))
         try:
-            return self._directory[_path].isdir()
+            return self._directory_entries[_path].isdir()
         except KeyError:
-            return any(isbase(_path, name) for name in self._directory)
+            return any(isbase(_path, name) for name in self._directory_entries)
 
     def isfile(self, path):
         _path = relpath(self.validatepath(path))
         try:
-            return self._directory[_path].isfile()
+            return self._directory_entries[_path].isfile()
         except KeyError:
             return False
 
@@ -384,7 +403,9 @@ class ReadTarFS(FS):
         if not self.gettype(path) is ResourceType.directory:
             raise errors.DirectoryExpected(path)
 
-        children = (frombase(_path, n) for n in self._directory if isbase(_path, n))
+        children = (
+            frombase(_path, n) for n in self._directory_entries if isbase(_path, n)
+        )
         content = (parts(child)[1] for child in children if relpath(child))
         return list(OrderedDict.fromkeys(content))
 
@@ -406,7 +427,7 @@ class ReadTarFS(FS):
             raise errors.ResourceReadOnly(path)
 
         try:
-            member = self._tar.getmember(self._encode(_path))
+            member = self._directory_entries[_path]
         except KeyError:
             six.raise_from(errors.ResourceNotFound(path), None)
 
@@ -437,27 +458,36 @@ class ReadTarFS(FS):
     def close(self):
         # type: () -> None
         super(ReadTarFS, self).close()
-        self._tar.close()
+        if hasattr(self, "_tar"):
+            self._tar.close()
 
     def isclosed(self):
         # type: () -> bool
         return self._tar.closed  # type: ignore
 
+    def geturl(self, path, purpose="download"):
+        # type: (Text, Text) -> Text
+        if purpose == "fs" and isinstance(self._file, six.string_types):
+            quoted_file = url_quote(self._file)
+            quoted_path = url_quote(path)
+            return "tar://{}!/{}".format(quoted_file, quoted_path)
+        else:
+            raise NoURL(path, purpose)
+
 
 if __name__ == "__main__":  # pragma: no cover
     from fs.tree import render
-    from fs.opener import open_fs
 
     with TarFS("tests.tar") as tar_fs:
         print(tar_fs.listdir("/"))
         print(tar_fs.listdir("/tests/"))
-        print(tar_fs.gettext("tests/ttt/settings.ini"))
+        print(tar_fs.readtext("tests/ttt/settings.ini"))
         render(tar_fs)
         print(tar_fs)
         print(repr(tar_fs))
 
     with TarFS("TarFS.tar", write=True) as tar_fs:
         tar_fs.makedirs("foo/bar")
-        tar_fs.settext("foo/bar/baz.txt", "Hello, World")
+        tar_fs.writetext("foo/bar/baz.txt", "Hello, World")
         print(tar_fs)
         print(repr(tar_fs))
