@@ -12,15 +12,13 @@ from collections import defaultdict
 from collections import deque
 from collections import namedtuple
 
-import six
-
 from ._repr import make_repr
 from .errors import FSError
 from .path import abspath
 from .path import combine
 from .path import normpath
 
-if False:  # typing.TYPE_CHECKING
+if typing.TYPE_CHECKING:
     from typing import (
         Any,
         Callable,
@@ -295,7 +293,7 @@ class Walker(object):
                 yield info
         except FSError as error:
             if not self.on_error(dir_path, error):
-                six.reraise(type(error), error)
+                raise
 
     def walk(
         self,
